@@ -169,14 +169,14 @@ impl GenerateCI {
         bridge_model: &BridgeModel,
         sdist: bool,
     ) -> Result<String> {
-        let is_abi3 = matches!(bridge_model, BridgeModel::BindingsAbi3(..));
+        let is_abi3 = matches!(bridge_model, BridgeModel::BindingsAbi3 { .. });
         let is_bin = bridge_model.is_bin();
         let setup_python = self.pytest
             || matches!(
                 bridge_model,
                 BridgeModel::Bin(Some(_))
                     | BridgeModel::Bindings { .. }
-                    | BridgeModel::BindingsAbi3(..)
+                    | BridgeModel::BindingsAbi3 { .. }
                     | BridgeModel::Cffi
                     | BridgeModel::UniFfi
             );
@@ -373,7 +373,7 @@ jobs:\n",
         with:
           target: ${{{{ matrix.platform.target }}}}
           args: --release --out dist{maturin_args}
-          sccache: 'true'
+          sccache: ${{{{ !startsWith(github.ref, 'refs/tags/') }}}}
 "
             ));
             let maturin_action_args = match platform {
@@ -393,7 +393,7 @@ jobs:\n",
         with:
           target: ${{{{ matrix.platform.target }}}}
           args: --release --out dist{maturin_args} -i python3.13t
-          sccache: 'true'
+          sccache: ${{{{ !startsWith(github.ref, 'refs/tags/') }}}}
 "
                 ));
                 if !maturin_action_args.is_empty() {
@@ -729,7 +729,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -760,7 +760,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -788,7 +788,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -814,7 +814,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -868,7 +868,18 @@ mod tests {
     #[test]
     fn test_generate_github_abi3() {
         let conf = GenerateCI::default()
-            .generate_github("example", &BridgeModel::BindingsAbi3(3, 7), false)
+            .generate_github(
+                "example",
+                &BridgeModel::BindingsAbi3 {
+                    bindings: Bindings {
+                        name: "pyo3".to_string(),
+                        version: Version::new(0, 23, 0),
+                    },
+                    major: 3,
+                    minor: 7,
+                },
+                false,
+            )
             .unwrap()
             .lines()
             .skip(5)
@@ -918,14 +929,14 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -956,14 +967,14 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -991,13 +1002,13 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1023,13 +1034,13 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1071,7 +1082,18 @@ mod tests {
             skip_attestation: true,
             ..Default::default()
         }
-        .generate_github("example", &BridgeModel::BindingsAbi3(3, 7), false)
+        .generate_github(
+            "example",
+            &BridgeModel::BindingsAbi3 {
+                bindings: Bindings {
+                    name: "pyo3".to_string(),
+                    version: Version::new(0, 23, 0),
+                },
+                major: 3,
+                minor: 7,
+            },
+            false,
+        )
         .unwrap()
         .lines()
         .skip(5)
@@ -1121,14 +1143,14 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1159,14 +1181,14 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1194,13 +1216,13 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1226,13 +1248,13 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1327,7 +1349,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter --zig
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1383,7 +1405,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1441,7 +1463,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1477,7 +1499,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1586,7 +1608,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1614,7 +1636,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1638,7 +1660,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1661,7 +1683,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
