@@ -10,7 +10,6 @@ use anyhow::ensure;
 use anyhow::{anyhow, bail, Context, Result};
 use cargo_options::heading;
 use fs_err as fs;
-use pep508_rs::{MarkerExpression, MarkerOperator, MarkerTree, MarkerValue};
 use regex::Regex;
 use std::path::Path;
 use std::path::PathBuf;
@@ -38,8 +37,53 @@ impl InstallBackend {
         }
     }
 
-    fn is_pip(&self) -> bool {
-        matches!(self, InstallBackend::Pip { .. })
+    fn version(&self, python_path: &Path) -> Result<semver::Version> {
+        let mut cmd = self.make_command(python_path);
+        let output = cmd
+            .arg("--version")
+            .output()
+            .context("failed to get version of install backend")?;
+        ensure!(
+            output.status.success(),
+            "failed to get version of install backend"
+        );
+        let stdout = str::from_utf8(&output.stdout)?;
+        let re = match self {
+            InstallBackend::Pip { .. } => Regex::new(r"pip ([\w\.]+).*"),
+            InstallBackend::Uv { .. } => Regex::new(r"uv-pip ([\w\.]+).*"),
+        };
+        if let Some(captures) = re.expect("regex should be valid").captures(stdout) {
+            Ok(semver::Version::parse(&captures[1])
+                .with_context(|| format!("failed to parse semver from {:?}", stdout))?)
+        } else {
+            bail!("failed to parse version from {:?}", stdout);
+        }
+    }
+
+    /// check whether this install backend supports `show --files`. Returns Ok(()) if it does.
+    fn check_supports_show_files(&self, python_path: &Path) -> Result<()> {
+        match self {
+            InstallBackend::Pip { .. } => Ok(()),
+            InstallBackend::Uv { .. } => {
+                // https://github.com/astral-sh/uv/releases/tag/0.4.25
+                let version = self.version(python_path)?;
+                if version < semver::Version::new(0, 4, 25) {
+                    bail!(
+                        "uv >= 0.4.25 is required for `show --files`. Version {} was found.",
+                        version
+                    );
+                }
+                Ok(())
+            }
+        }
+    }
+
+    fn stderr_indicates_problem(&self) -> bool {
+        match self {
+            InstallBackend::Pip { .. } => true,
+            // `uv pip install` sends regular logs to stderr, not just errors
+            InstallBackend::Uv { .. } => false,
+        }
     }
 
     fn make_command(&self, python_path: &Path) -> Command {
@@ -80,6 +124,25 @@ fn find_uv_bin() -> Result<(PathBuf, Vec<&'static str>)> {
     }
 }
 
+/// Detect the Python uv package
+fn find_uv_python(python_path: &Path) -> Result<(PathBuf, Vec<&'static str>)> {
+    let output = Command::new(python_path)
+        .args(["-m", "uv", "--version"])
+        .output()?;
+    if output.status.success() {
+        let version_str =
+            str::from_utf8(&output.stdout).context("`uv --version` didn't return utf8 output")?;
+        debug!(version = %version_str, "Found Python uv module");
+        Ok((python_path.to_path_buf(), vec!["-m", "uv"]))
+    } else {
+        bail!(
+            "`{} -m uv --version` failed with status: {}",
+            python_path.display(),
+            output.status
+        );
+    }
+}
+
 fn check_pip_exists(python_path: &Path, pip_path: Option<&PathBuf>) -> Result<()> {
     let output = if let Some(pip_path) = pip_path {
         Command::new(pip_path).args(["--version"]).output()?
@@ -98,22 +161,15 @@ fn check_pip_exists(python_path: &Path, pip_path: Option<&PathBuf>) -> Result<()
     }
 }
 
-/// Detect the Python uv package
-fn find_uv_python(python_path: &Path) -> Result<(PathBuf, Vec<&'static str>)> {
-    let output = Command::new(python_path)
-        .args(["-m", "uv", "--version"])
-        .output()?;
-    if output.status.success() {
-        let version_str =
-            str::from_utf8(&output.stdout).context("`uv --version` didn't return utf8 output")?;
-        debug!(version = %version_str, "Found Python uv module");
-        Ok((python_path.to_path_buf(), vec!["-m", "uv"]))
-    } else {
-        bail!(
-            "`{} -m uv --version` failed with status: {}",
-            python_path.display(),
-            output.status
-        );
+/// Check if a virtualenv is created by uv by reading pyvenv.cfg
+fn is_uv_venv(venv_dir: &Path) -> bool {
+    let pyvenv_cfg = venv_dir.join("pyvenv.cfg");
+    if !pyvenv_cfg.exists() {
+        return false;
+    }
+    match fs::read_to_string(&pyvenv_cfg) {
+        Ok(content) => content.contains("\nuv = "),
+        Err(_) => false,
     }
 }
 
@@ -167,44 +223,42 @@ pub struct DevelopOptions {
 fn install_dependencies(
     build_context: &BuildContext,
     extras: &[String],
-    interpreter: &PythonInterpreter,
+    python: &Path,
+    venv_dir: &Path,
     install_backend: &InstallBackend,
 ) -> Result<()> {
-    if !build_context.metadata23.requires_dist.is_empty() {
+    if !build_context.metadata24.requires_dist.is_empty() {
+        let mut extra_names = Vec::with_capacity(extras.len());
+        for extra in extras {
+            extra_names.push(
+                pep508_rs::ExtraName::new(extra.clone())
+                    .with_context(|| format!("invalid extra name: {extra}"))?,
+            );
+        }
         let mut args = vec!["install".to_string()];
-        args.extend(build_context.metadata23.requires_dist.iter().map(|x| {
+        args.extend(build_context.metadata24.requires_dist.iter().map(|x| {
             let mut pkg = x.clone();
-            // Remove extra marker to make it installable with pip
-            // Keep in sync with `Metadata21::merge_pyproject_toml()`!
-            for extra in extras {
-                pkg.marker = pkg.marker.and_then(|marker| -> Option<MarkerTree> {
-                    match marker.clone() {
-                        MarkerTree::Expression(MarkerExpression {
-                            l_value: MarkerValue::Extra,
-                            operator: MarkerOperator::Equal,
-                            r_value: MarkerValue::QuotedString(extra_value),
-                        }) if &extra_value == extra => None,
-                        MarkerTree::And(and) => match &*and {
-                            [existing, MarkerTree::Expression(MarkerExpression {
-                                l_value: MarkerValue::Extra,
-                                operator: MarkerOperator::Equal,
-                                r_value: MarkerValue::QuotedString(extra_value),
-                            })] if extra_value == extra => Some(existing.clone()),
-                            _ => Some(marker),
-                        },
-                        _ => Some(marker),
-                    }
-                });
-            }
+            // Remove extra marker to make it installable with pip:
+            //
+            // * ` and extra == 'EXTRA_NAME'`
+            // * `; extra == 'EXTRA_NAME'`
+            //
+            // Keep in sync with `Metadata23::merge_pyproject_toml()`
+            pkg.marker = pkg.marker.simplify_extras(&extra_names);
             pkg.to_string()
         }));
         let status = install_backend
-            .make_command(&interpreter.executable)
+            .make_command(python)
             .args(&args)
+            .env("VIRTUAL_ENV", venv_dir)
             .status()
-            .context("Failed to run pip install")?;
+            .with_context(|| format!("Failed to run {} install", install_backend.name()))?;
         if !status.success() {
-            bail!(r#"pip install finished with "{}""#, status)
+            bail!(
+                r#"{} install finished with "{}""#,
+                install_backend.name(),
+                status
+            )
         }
     }
     Ok(())
@@ -222,6 +276,7 @@ fn install_wheel(
     let output = cmd
         .args(["install", "--no-deps", "--force-reinstall"])
         .arg(dunce::simplified(wheel_filename))
+        .env("VIRTUAL_ENV", venv_dir)
         .output()
         .context(format!(
             "{} install failed (ran {:?} with {:?})",
@@ -240,15 +295,17 @@ fn install_wheel(
             String::from_utf8_lossy(&output.stderr).trim(),
         );
     }
-    // uv pip install sends logs to stderr thus only print this warning for pip
-    if !output.stderr.is_empty() && install_backend.is_pip() {
+    if !output.stderr.is_empty() && install_backend.stderr_indicates_problem() {
         eprintln!(
-            "⚠️ Warning: pip raised a warning running {:?}:\n{}",
+            "⚠️ Warning: {} raised a warning running {:?}:\n{}",
+            install_backend.name(),
             &cmd.get_args().collect::<Vec<_>>(),
             String::from_utf8_lossy(&output.stderr).trim(),
         );
     }
-    fix_direct_url(build_context, python, install_backend)?;
+    if let Err(err) = configure_as_editable(build_context, python, install_backend) {
+        eprintln!("⚠️ Warning: failed to set package as editable: {}", err);
+    }
     Ok(())
 }
 
@@ -260,14 +317,15 @@ fn install_wheel(
 /// correct URL, however when a maturin package is installed with `maturin develop`, the URL is
 /// set to the path to the temporary wheel file created during installation.
 #[instrument(skip_all)]
-fn fix_direct_url(
+fn configure_as_editable(
     build_context: &BuildContext,
     python: &Path,
     install_backend: &InstallBackend,
 ) -> Result<()> {
-    println!("✏️  Setting installed package as editable");
+    println!("✏️ Setting installed package as editable");
+    install_backend.check_supports_show_files(python)?;
     let mut cmd = install_backend.make_command(python);
-    let cmd = cmd.args(["show", "--files", &build_context.metadata23.name]);
+    let cmd = cmd.args(["show", "--files", &build_context.metadata24.name]);
     debug!("running {:?}", cmd);
     let output = cmd.output()?;
     ensure!(output.status.success(), "failed to list package files");
@@ -355,15 +413,39 @@ pub fn develop(develop_options: DevelopOptions, venv_dir: &Path) -> Result<()> {
         .editable(true)
         .build()?;
 
+    // Ensure that version information is present, https://github.com/PyO3/maturin/issues/2416
+    if build_context
+        .pyproject_toml
+        .as_ref()
+        .is_some_and(|p| !p.warn_invalid_version_info())
+    {
+        bail!("Cannot build without valid version information. \
+               You need to specify either `project.version` or `project.dynamic = [\"version\"]` in pyproject.toml.");
+    }
+
     let interpreter =
         PythonInterpreter::check_executable(&python, &target, build_context.bridge())?.ok_or_else(
             || anyhow!("Expected `python` to be a python interpreter inside a virtualenv ಠ_ಠ"),
         )?;
 
-    let install_backend = if uv {
-        let (uv_path, uv_args) = find_uv_python(&interpreter.executable)
-            .or_else(|_| find_uv_bin())
-            .context("Failed to find uv")?;
+    let uv_venv = is_uv_venv(venv_dir);
+    let uv_info = if uv || uv_venv {
+        match find_uv_python(&interpreter.executable).or_else(|_| find_uv_bin()) {
+            Ok(uv_info) => Some(Ok(uv_info)),
+            Err(e) => {
+                if uv {
+                    Some(Err(e))
+                } else {
+                    // Ignore error and try pip instead if it's a uv venv but `--uv` is not specified
+                    None
+                }
+            }
+        }
+    } else {
+        None
+    };
+    let install_backend = if let Some(uv_info) = uv_info {
+        let (uv_path, uv_args) = uv_info?;
         InstallBackend::Uv {
             path: uv_path,
             args: uv_args,
@@ -376,7 +458,7 @@ pub fn develop(develop_options: DevelopOptions, venv_dir: &Path) -> Result<()> {
         }
     };
 
-    install_dependencies(&build_context, &extras, &interpreter, &install_backend)?;
+    install_dependencies(&build_context, &extras, &python, venv_dir, &install_backend)?;
 
     let wheels = build_context.build_wheels()?;
     if !skip_install {
@@ -390,7 +472,7 @@ pub fn develop(develop_options: DevelopOptions, venv_dir: &Path) -> Result<()> {
             )?;
             eprintln!(
                 "🛠 Installed {}-{}",
-                build_context.metadata23.name, build_context.metadata23.version
+                build_context.metadata24.name, build_context.metadata24.version
             );
         }
     }
