@@ -793,6 +793,28 @@ fn handle_cffi_call_result(
     }
 }
 
+// Extract the shared object from a AIX big library archive
+fn unpack_big_archive(target: &Target, artifact: &Path, temp_dir_path: &Path) -> Result<PathBuf> {
+    // Newer rust generates archived dylibs on AIX, as shared
+    // libraries are typically archived on the platform.
+    if target.cross_compiling() {
+        bail!("can't unpack big archive format while cross_compiling")
+    }
+    debug!("Unpacking archive {}", artifact.display());
+    let mut ar_command = Command::new("ar");
+    ar_command
+        .current_dir(temp_dir_path)
+        .arg("-X64")
+        .arg("x")
+        .arg(artifact);
+    let status = ar_command.status().expect("Failed to run ar");
+    if !status.success() {
+        bail!(r#"ar finished with "{}": `{:?}`"#, status, ar_command,)
+    }
+    let unpacked_artifact = temp_dir_path.join(artifact.with_extension("so").file_name().unwrap());
+    Ok(unpacked_artifact)
+}
+
 /// Copies the shared library into the module, which is the only extra file needed with bindings
 #[allow(clippy::too_many_arguments)]
 #[instrument(skip_all)]
@@ -825,6 +847,28 @@ pub fn write_bindings_module(
         python_interpreter.get_library_name(ext_name)
     };
 
+    let artifact_is_big_ar =
+        target.is_aix() && artifact.extension().unwrap_or(OsStr::new(" ")) == OsStr::new("a");
+    let temp_dir = if artifact_is_big_ar {
+        Some(tempfile::tempdir()?)
+    } else {
+        None
+    };
+    let artifact_buff = if artifact_is_big_ar {
+        Some(unpack_big_archive(
+            target,
+            artifact,
+            temp_dir.as_ref().unwrap().path(),
+        )?)
+    } else {
+        None
+    };
+    let artifact = if artifact_is_big_ar {
+        artifact_buff.as_ref().unwrap()
+    } else {
+        artifact
+    };
+
     if !editable {
         write_python_part(writer, project_layout, pyproject_toml)
             .context("Failed to add the python module to the package")?;
@@ -1117,12 +1161,7 @@ fn generate_uniffi_bindings(
 
     let py_bindings = fs::read_dir(&binding_dir)?
         .flatten()
-        .filter(|file| {
-            file.path()
-                .extension()
-                .and_then(std::ffi::OsStr::to_str)
-                .map_or(false, |ext| ext == "py")
-        })
+        .filter(|file| file.path().extension().and_then(std::ffi::OsStr::to_str) == Some("py"))
         .map(|file| {
             file.path()
                 .file_stem()
@@ -1446,7 +1485,11 @@ pub fn write_dist_info(
 /// to save or generate the data in other places
 ///
 /// See https://peps.python.org/pep-0427/#file-contents
-pub fn add_data(writer: &mut impl ModuleWriter, data: Option<&Path>) -> Result<()> {
+pub fn add_data(
+    writer: &mut impl ModuleWriter,
+    metadata24: &Metadata24,
+    data: Option<&Path>,
+) -> Result<()> {
     let possible_data_dir_names = ["data", "scripts", "headers", "purelib", "platlib"];
     if let Some(data) = data {
         for subdir in fs::read_dir(data).context("Failed to read data dir")? {
@@ -1474,7 +1517,9 @@ pub fn add_data(writer: &mut impl ModuleWriter, data: Option<&Path>) -> Result<(
                     let mode = file.metadata()?.permissions().mode();
                     #[cfg(not(unix))]
                     let mode = 0o644;
-                    let relative = file.path().strip_prefix(data.parent().unwrap()).unwrap();
+                    let relative = metadata24
+                        .get_data_dir()
+                        .join(file.path().strip_prefix(data).unwrap());
 
                     if file.path_is_symlink() {
                         // Copy the actual file contents, not the link, so that you can create a
