@@ -1,7 +1,7 @@
 pub use self::config::InterpreterConfig;
 use crate::auditwheel::PlatformTag;
 use crate::{BridgeModel, BuildContext, Target};
-use anyhow::{bail, format_err, Context, Result};
+use anyhow::{bail, ensure, format_err, Context, Result};
 use pep440_rs::{Version, VersionSpecifiers};
 use regex::Regex;
 use serde::Deserialize;
@@ -20,6 +20,7 @@ mod config;
 /// version and abi as json through stdout
 const GET_INTERPRETER_METADATA: &str = include_str!("get_interpreter_metadata.py");
 pub const MINIMUM_PYTHON_MINOR: usize = 7;
+pub const MINIMUM_PYPY_MINOR: usize = 8;
 /// Be liberal here to include preview versions
 pub const MAXIMUM_PYTHON_MINOR: usize = 13;
 pub const MAXIMUM_PYPY_MINOR: usize = 10;
@@ -427,7 +428,19 @@ fn fun_with_abiflags(
         Ok("".to_string())
     } else if message.system == "windows" {
         if matches!(message.abiflags.as_deref(), Some("") | None) {
-            Ok("".to_string())
+            // windows has a few annoying cases, but its abiflags in sysconfig always empty
+            // python <= 3.7 has "m"
+            if message.minor <= 7 {
+                Ok("m".to_string())
+            } else if message.gil_disabled {
+                ensure!(
+                    message.minor >= 13,
+                    "gil_disabled is only available in python 3.13+ ಠ_ಠ"
+                );
+                Ok("t".to_string())
+            } else {
+                Ok("".to_string())
+            }
         } else {
             bail!("A python 3 interpreter on Windows does not define abiflags in its sysconfig ಠ_ಠ")
         }
@@ -499,23 +512,13 @@ impl PythonInterpreter {
         } else {
             match self.interpreter_kind {
                 InterpreterKind::CPython => {
-                    if target.is_unix() {
-                        format!(
-                            "cp{major}{minor}-cp{major}{minor}{abiflags}-{platform}",
-                            major = self.major,
-                            minor = self.minor,
-                            abiflags = self.abiflags,
-                            platform = platform
-                        )
-                    } else {
-                        // On windows the abiflags are missing, but this seems to work
-                        format!(
-                            "cp{major}{minor}-none-{platform}",
-                            major = self.major,
-                            minor = self.minor,
-                            platform = platform
-                        )
-                    }
+                    format!(
+                        "cp{major}{minor}-cp{major}{minor}{abiflags}-{platform}",
+                        major = self.major,
+                        minor = self.minor,
+                        abiflags = self.abiflags,
+                        platform = platform
+                    )
                 }
                 InterpreterKind::PyPy => {
                     // pypy uses its version as part of the ABI, e.g.
@@ -727,7 +730,17 @@ impl PythonInterpreter {
     pub fn find_by_target(
         target: &Target,
         requires_python: Option<&VersionSpecifiers>,
+        bridge: Option<&BridgeModel>,
     ) -> Vec<PythonInterpreter> {
+        let min_python_minor = bridge
+            .map(|bridge| bridge.minimal_python_minor_version())
+            .unwrap_or(MINIMUM_PYTHON_MINOR);
+        let min_pypy_minor = bridge
+            .map(|bridge| bridge.minimal_pypy_minor_version())
+            .unwrap_or(MINIMUM_PYPY_MINOR);
+        let supports_free_threaded = bridge
+            .map(|bridge| bridge.supports_free_threaded())
+            .unwrap_or(false);
         InterpreterConfig::lookup_target(target)
             .into_iter()
             .filter_map(|config| match requires_python {
@@ -742,6 +755,30 @@ impl PythonInterpreter {
                 }
                 None => Some(Self::from_config(config)),
             })
+            .filter_map(|config| match config.interpreter_kind {
+                InterpreterKind::CPython => {
+                    if config.minor >= min_python_minor {
+                        Some(config)
+                    } else {
+                        None
+                    }
+                }
+                InterpreterKind::PyPy => {
+                    if config.minor >= min_pypy_minor {
+                        Some(config)
+                    } else {
+                        None
+                    }
+                }
+                InterpreterKind::GraalPy => Some(config),
+            })
+            .filter_map(|config| {
+                if config.gil_disabled && !supports_free_threaded {
+                    None
+                } else {
+                    Some(config)
+                }
+            })
             .collect()
     }
 
@@ -756,11 +793,10 @@ impl PythonInterpreter {
         bridge: &BridgeModel,
         requires_python: Option<&VersionSpecifiers>,
     ) -> Result<Vec<PythonInterpreter>> {
-        let min_python_minor = match bridge {
-            BridgeModel::Bindings(_, minor) | BridgeModel::Bin(Some((_, minor))) => *minor,
-            _ => MINIMUM_PYTHON_MINOR,
-        };
+        let min_python_minor = bridge.minimal_python_minor_version();
+        let min_pypy_minor = bridge.minimal_pypy_minor_version();
         let executables = if target.is_windows() {
+            // TOFIX: add PyPy support to Windows
             find_all_windows(target, min_python_minor, requires_python)?
         } else {
             let mut executables: Vec<String> = (min_python_minor..=MAXIMUM_PYTHON_MINOR)
@@ -779,7 +815,7 @@ impl PythonInterpreter {
                 || bridge.is_bindings("pyo3-ffi")
             {
                 executables.extend(
-                    (min_python_minor..=MAXIMUM_PYPY_MINOR)
+                    (min_pypy_minor..=MAXIMUM_PYPY_MINOR)
                         .filter(|minor| {
                             requires_python
                                 .map(|requires_python| {
@@ -988,26 +1024,131 @@ fn calculate_abi_tag(ext_suffix: &str) -> Option<String> {
 
 #[cfg(test)]
 mod tests {
+    use crate::Bindings;
+    use expect_test::expect;
+
     use super::*;
 
     #[test]
     fn test_find_interpreter_by_target() {
         let target =
             Target::from_target_triple(Some("x86_64-unknown-linux-gnu".to_string())).unwrap();
-        let pythons = PythonInterpreter::find_by_target(&target, None);
-        assert_eq!(pythons.len(), 12);
+        let pythons = PythonInterpreter::find_by_target(&target, None, None)
+            .iter()
+            .map(ToString::to_string)
+            .collect::<Vec<_>>();
+        let expected = expect![[r#"
+            [
+                "CPython 3.7m",
+                "CPython 3.8",
+                "CPython 3.9",
+                "CPython 3.10",
+                "CPython 3.11",
+                "CPython 3.12",
+                "CPython 3.13",
+                "PyPy 3.8",
+                "PyPy 3.9",
+                "PyPy 3.10",
+            ]
+        "#]];
+        expected.assert_debug_eq(&pythons);
+
+        // pyo3 0.23+ should find CPython 3.13t
+        let pythons = PythonInterpreter::find_by_target(
+            &target,
+            None,
+            Some(&BridgeModel::Bindings(Bindings {
+                name: "pyo3".to_string(),
+                version: semver::Version::new(0, 23, 0),
+            })),
+        )
+        .iter()
+        .map(ToString::to_string)
+        .collect::<Vec<_>>();
+        let expected = expect![[r#"
+            [
+                "CPython 3.7m",
+                "CPython 3.8",
+                "CPython 3.9",
+                "CPython 3.10",
+                "CPython 3.11",
+                "CPython 3.12",
+                "CPython 3.13",
+                "CPython 3.13t",
+                "PyPy 3.9",
+                "PyPy 3.10",
+            ]
+        "#]];
+        expected.assert_debug_eq(&pythons);
 
         let pythons = PythonInterpreter::find_by_target(
             &target,
             Some(&VersionSpecifiers::from_str(">=3.8").unwrap()),
-        );
-        assert_eq!(pythons.len(), 10);
+            None,
+        )
+        .iter()
+        .map(ToString::to_string)
+        .collect::<Vec<_>>();
+        let expected = expect![[r#"
+            [
+                "CPython 3.8",
+                "CPython 3.9",
+                "CPython 3.10",
+                "CPython 3.11",
+                "CPython 3.12",
+                "CPython 3.13",
+                "PyPy 3.8",
+                "PyPy 3.9",
+                "PyPy 3.10",
+            ]
+        "#]];
+        expected.assert_debug_eq(&pythons);
 
         let pythons = PythonInterpreter::find_by_target(
             &target,
             Some(&VersionSpecifiers::from_str(">=3.10").unwrap()),
-        );
-        assert_eq!(pythons.len(), 6);
+            None,
+        )
+        .iter()
+        .map(ToString::to_string)
+        .collect::<Vec<_>>();
+        let expected = expect![[r#"
+            [
+                "CPython 3.10",
+                "CPython 3.11",
+                "CPython 3.12",
+                "CPython 3.13",
+                "PyPy 3.10",
+            ]
+        "#]];
+        expected.assert_debug_eq(&pythons);
+
+        let pythons = PythonInterpreter::find_by_target(
+            &target,
+            Some(&VersionSpecifiers::from_str(">=3.8").unwrap()),
+            Some(&BridgeModel::Bindings(Bindings {
+                name: "pyo3".to_string(),
+                version: semver::Version::new(0, 23, 0),
+            })),
+        )
+        .iter()
+        .map(ToString::to_string)
+        .collect::<Vec<_>>();
+        // should exclude PyPy < 3.9
+        let expected = expect![[r#"
+            [
+                "CPython 3.8",
+                "CPython 3.9",
+                "CPython 3.10",
+                "CPython 3.11",
+                "CPython 3.12",
+                "CPython 3.13",
+                "CPython 3.13t",
+                "PyPy 3.9",
+                "PyPy 3.10",
+            ]
+        "#]];
+        expected.assert_debug_eq(&pythons);
     }
 
     #[test]
